package com.covas.Resources;

import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.transaction.Transactional;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.CookieParam;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.PATCH;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.covas.Entity.CommentEntity;
import com.covas.Entity.MessageEntity;
import com.covas.Entity.PublisherEntity;
import com.covas.Entity.UsersEntity;
import com.covas.Json.MessageByUser;

import io.quarkus.panache.common.Page;
import io.quarkus.panache.common.Parameters;

import org.eclipse.microprofile.jwt.Claims;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.jboss.logging.Logger;
import org.postgresql.shaded.com.ongres.scram.common.bouncycastle.base64.Base64;

@Produces(MediaType.APPLICATION_JSON)
@Path("message")
public class MessageRessources {
    private static final Logger LOGGER = Logger.getLogger(UsersRessources.class);
    @Inject
    JsonWebToken jwt;

    /// Function
    private Boolean checkUserCookie(String userCookie, UsersEntity users) {
        if ((userCookie == null) || (users == null)) {
            return false;
        }
        String name = new String(Base64.decode(userCookie), StandardCharsets.UTF_8);
        if (!name.equals(users.pseudo) && (users.status != 1)) {
            return false;
        }
        return true;
    }

    private Response.Status getResponseCheck(SecurityContext ctx, String userCookie, UsersEntity users) {
        if (!ctx.getUserPrincipal().getName().equals(jwt.getName())) {
            return Response.Status.INTERNAL_SERVER_ERROR;
        }
        if (!checkUserCookie(userCookie, users)) {
            return Response.Status.FORBIDDEN;
        }
        return Response.Status.OK;
    }

    /// Appel HTTP

    /// GET
    @GET
    @RolesAllowed("Admin")
    public Response getMessages(@CookieParam("user") String userCookie, @Context SecurityContext ctx, 
    @QueryParam("page") Integer page, @QueryParam("nbPages") Integer nbPages, 
    @QueryParam("status") Short status,
    @QueryParam("search") String search, 
    @QueryParam("uuid") String uuid) {
        if(nbPages == null){
            nbPages = 20;
        }
        if(page == null){
            page = 0;
        }
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status statusHttp = getResponseCheck(ctx, userCookie, user);
        Response responseHttp = Response.status(statusHttp).build();
        if (statusHttp.equals(Response.Status.OK)) {
            List<MessageEntity> listMessages = MessageEntity.findAll().page(Page.of(page, nbPages)).list();

        
            responseHttp = Response.ok(listMessages).build();
            if(uuid != null){
                MessageEntity messageEntity= MessageEntity.findById(UUID.fromString(uuid));
                responseHttp = Response.ok(messageEntity).build();
            }
          
            if(search != null){
                List<MessageEntity> messagesList = MessageEntity.find("#Message.bySearch", Parameters.with("content", search)).page(Page.of(page, nbPages)).list();
               
                responseHttp = Response.ok(messagesList).build();

            }
        }
        return responseHttp;
    }

    @GET
    @RolesAllowed("Admin")
    @Path("count")
    public Response getCount(@CookieParam("user") String userCookie, @Context SecurityContext ctx){
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        Response responseHttp = Response.status(status).build();
        if (status.equals(Response.Status.OK)){
            responseHttp = Response.ok(MessageEntity.count()).build();
        }
        return responseHttp;
    }

    @GET
    @RolesAllowed("Admin")
    @Path("{id}")
    public Response getSingleMessage(@PathParam("id") String id, @CookieParam("user") String userCookie,
            @Context SecurityContext ctx) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        Response responseHttp = Response.status(status).build();
        if (status.equals(Response.Status.OK)) {
            UUID uid = UUID.fromString(id);
            MessageEntity message = MessageEntity.findById(uid);
            responseHttp = Response.status(Response.Status.NOT_FOUND).build();
            if (message != null) {
                responseHttp = Response.ok(message).build();
            }

        }
        return responseHttp;
    }

    @GET
    @RolesAllowed("User")
    @Path("info")
    public Response getInfoMessage(@Context SecurityContext ctx, @CookieParam("user") String userCookie) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        if (status.equals(Response.Status.OK)) {
            if (user == null) {
                status = Response.Status.NOT_FOUND;
            }
            if (!checkUserCookie(userCookie, user)) {
                status = Response.Status.FORBIDDEN;
            }
        }
        Response responseHttp = Response.status(status).build();
        
        if (status.equals(Response.Status.OK)) {
            responseHttp = Response.status(status).entity(new MessageByUser(user.pseudo, user.message))
                    .build();
        }
        return responseHttp;
    }


    
    // PUT
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @RolesAllowed({"Admin", "User"})
    @Path("{id}")
    @Transactional
    public Response createMessage(@Context SecurityContext ctx, @CookieParam("user") String userCookie, @PathParam("id") String id, MessageEntity message) {

            UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
            UsersEntity user = UsersEntity.findById(kid);
            Response.Status status = getResponseCheck(ctx, userCookie, user);
            MessageEntity newMessage = new MessageEntity();
            if (status.equals(Response.Status.OK)) {
                newMessage.content = message.content;
                newMessage.conversation = message.conversation;
                newMessage.users = user;
                newMessage.created_at = LocalDateTime.now();
                newMessage.updated_at = LocalDateTime.now();
                newMessage.persist();
                if (newMessage.isPersistent()) {
                    status = Response.Status.CREATED;
                } else {
                    status = Response.Status.NO_CONTENT;
                }
            }            
        return Response.status(status).entity(newMessage).build();
    }


    // DELETE
    @DELETE
    @Path("{id}")
    @RolesAllowed({"Admin", "User"})
    @Transactional
    public Response changeStatusSingleMessageToDelete(@Context SecurityContext ctx, @CookieParam("user") String userCookie,
            @PathParam("id") String id) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        if (status.equals(Response.Status.OK)) {
            MessageEntity singleMessage= MessageEntity.find("id", UUID.fromString(id)).firstResult();
            if (singleMessage == null) {
                status = Response.Status.NOT_FOUND;
            } else {
                singleMessage.status = -1;
                singleMessage.updated_at = LocalDateTime.now();
                singleMessage.deleted_at = LocalDateTime.now();
                singleMessage.persist();
                if (!singleMessage.isPersistent()) {
                    status = Response.Status.NOT_MODIFIED;
                }
            }
        }
        return Response.status(status).build();
    }

    @DELETE
    @Path("/disable/{id}")
    @RolesAllowed("Admin")
    @Transactional
    public Response changeStatusSingleMessageToDisable(@Context SecurityContext ctx, @CookieParam("user") String userCookie,
            @PathParam("id") String id) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        if (status.equals(Response.Status.OK)) {
            MessageEntity singleMessage = MessageEntity.find("id", UUID.fromString(id)).firstResult();
            if (singleMessage == null) {
                status = Response.Status.NOT_FOUND;
            } else {
                singleMessage.status = 0;
                singleMessage.updated_at = LocalDateTime.now();
                singleMessage.deleted_at = LocalDateTime.now();
                singleMessage.persist();
                if (!singleMessage.isPersistent()) {
                    status = Response.Status.NOT_MODIFIED;
                }
            }
        }
        return Response.status(status).build();
    }

    // PATCH
    @PATCH
    @RolesAllowed({"Admin", "User"})
    @Consumes(MediaType.APPLICATION_JSON)
    @Transactional
    @Path("{id}")
    public Response updateMessageAdmin(@Context SecurityContext ctx, @CookieParam("user") String userCookie,
    MessageEntity message, @PathParam("id") String id) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);

        if (status.equals(Response.Status.OK)) {

            MessageEntity messageOrig = MessageEntity.findById(UUID.fromString(id));
            if (messageOrig == null) {
                status = Response.Status.NOT_FOUND;
            } else {
                messageOrig.content = message.content;
                
                
                messageOrig.updated_at = LocalDateTime.now();
                
                if(message.status == 1){
                    messageOrig.deleted_at = null;
                    messageOrig.status = 1;
                }
                messageOrig.persist();
                if (!messageOrig.isPersistent()) {
                    status = Response.Status.NOT_MODIFIED;
                }
            }
        }
        return Response.status(status).build();
    }



    @PATCH
    @RolesAllowed("Admin")
    @Consumes(MediaType.APPLICATION_JSON)
    @Transactional
    @Path("enable/{id}")
    public Response enableCommentAdmin(@Context SecurityContext ctx, @CookieParam("user") String userCookie, @PathParam("id") String id) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        if (status.equals(Response.Status.OK)) {

            MessageEntity messageOrig = MessageEntity.findById(UUID.fromString(id));
            if (messageOrig == null) {
                status = Response.Status.NOT_FOUND;
            } else {
                messageOrig.status = 1;
                messageOrig.persist();
                if (!messageOrig.isPersistent()) {
                    status = Response.Status.NOT_MODIFIED;
                }
            }
        }
        return Response.status(status).build();
    }
}
